#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Author: xuwei
Email: 18810079020@163.com
File: sqlite_op.py
Date: 2021/2/23 11:37 上午
'''

import sqlite3
import json
import numpy
import datetime
import pandas as pd
from datetime import date


##datetime.datetime is not JSON serializable 报错问题解决
class CJsonEncoder(json.JSONEncoder):
    def default(self, obj):
        if obj != obj:
            return None
        elif isinstance(obj, datetime.datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, date):
            return obj.strftime("%Y-%m-%d")
        elif isinstance(obj, numpy.integer):
            return int(obj)
        elif isinstance(obj, numpy.floating):
            return float(obj)
        elif isinstance(obj, numpy.ndarray):
            return obj.tolist()
        elif isinstance(obj, bytes):
            return str(obj)
        return json.JSONEncoder.default(self, obj)


class DataTypeToStr(object):
    @classmethod
    def data_type_to_str(cls, obj):
        if obj != obj:
            return None
        elif isinstance(obj, datetime.datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, numpy.datetime64):
            return pd.to_datetime(str(obj)).strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, date):
            return obj.strftime("%Y-%m-%d")
        elif isinstance(obj, numpy.integer) or isinstance(obj, numpy.floating) or isinstance(obj, int) \
                or isinstance(obj, float) or isinstance(obj, bytes):
            return str(obj)
        elif isinstance(obj, numpy.ndarray):
            return json.dumps(obj.tolist(), ensure_ascii=False, cls=CJsonEncoder)
        elif isinstance(obj, list):
            return json.dumps(obj, ensure_ascii=False, cls=CJsonEncoder)
        elif isinstance(obj, dict):
            return json.dumps(obj, ensure_ascii=False, cls=CJsonEncoder)
        else:
            return obj

    @classmethod
    def dict_values_type_etl_str(cls, dict_param):
        for key in dict_param:
            dict_param[key] = cls.data_type_to_str(dict_param[key])
        return dict_param

    @classmethod
    def list_dict_values_type_to_str(cls, list_dict):
        list_dict = list(map(cls.dict_values_type_etl_str, list_dict))
        return list_dict


class SqliteOp(object):
    def __init__(self, db_name):
        self.db = db_name

    # 数据库操作
    def operate_sql(self, sql):
        con = sqlite3.connect(self.db)
        try:
            cursor = con.cursor()
            cursor.execute(sql)
            con.commit()
        except Exception as e:
            con.rollback()
            raise e
        finally:
            con.close()

    # 主要用来insert into 多条数据
    def operate_execute_many(self, sql, data):
        con = sqlite3.connect(self.db)
        try:
            cursor = con.cursor()
            cursor.executemany(sql, data)
            con.commit()
        except Exception as e:
            con.rollback()
            raise e
        finally:
            con.close()

    # 此函数中df的columns或dict的key值必须和数据库中的字段一致
    def execute_many_write_data(self, data, table_name):
        """
        :param data: list or Dataframe
        :param table_name:
        :return:
        """
        if not isinstance(data, list):
            list_dict = data.to_dict('records')
        else:
            list_dict = data
        list_dict = DataTypeToStr.list_dict_values_type_to_str(list_dict)
        _columns_list = list_dict[0].keys()
        _columns_num = len(_columns_list)
        _columns = '`,`'.join(_columns_list)
        _columns = '`' + _columns + '`'
        _placeholder = ','.join(['?'] * _columns_num)
        _sql = 'insert or ignore into `%s` (' % table_name + _columns + ') values (' + _placeholder + ')'
        _list_tuple = [tuple(_dict.values()) for _dict in list_dict]
        self.operate_execute_many(_sql, _list_tuple)

    def sql_query(self, sql):
        con = sqlite3.connect(self.db)
        try:
            cursor = con.cursor()
            cursor.execute(sql)
            results = cursor.fetchall()
            return results
        except Exception as e:
            raise e
        finally:
            con.close()

if __name__ == '__main__':
    sqliteOp = SqliteOp('../goods.db')

    sql = """
    CREATE TABLE if not exists `goods_details` (
  `url` varchar(400) PRIMARY KEY  NOT NULL,
  `title` varchar(600) DEFAULT NULL,
  `pic` varchar(500) DEFAULT NULL,
  `source` varchar(100) DEFAULT NULL,
  `price` varchar(100)  DEFAULT NULL,
  `comment` varchar(100)  DEFAULT NULL,
  `commnet_url` varchar(500) DEFAULT NULL
);
    """
    sqliteOp.operate_sql(sql)


from gmssl.sm4 import CryptSM4, SM4_ENCRYPT, SM4_DECRYPT
import binascii
import base64


class SM4:
    """
    国密sm4加解密
    """

    def __init__(self):
        self.crypt_sm4 = CryptSM4()

    def str_to_hexStr(self, hex_str):
        """
        字符串转hex
        :param hex_str: 字符串
        :return: hex
        """
        hex_data = hex_str.encode('utf-8')
        str_bin = binascii.unhexlify(hex_data)
        return str_bin.decode('utf-8')

    def encrypt(self, encrypt_key, value) -> str:
        """
        国密sm4加密
        :param encrypt_key: sm4加密key
        :param value: 待加密的字符串
        :return: sm4加密后的hex值
        """
        crypt_sm4 = self.crypt_sm4
        crypt_sm4.set_key(encrypt_key.encode(), SM4_ENCRYPT)
        date_str = str(value)
        encrypt_value = crypt_sm4.crypt_ecb(date_str.encode())  # bytes类型
        # return encrypt_value.hex()
        return base64.b64encode(encrypt_value).decode()

    def decrypt(self, decrypt_key, encrypt_value: str):
        """
        国密sm4解密
        :param decrypt_key:sm4加密key
        :param encrypt_value: 待解密的hex值
        :return: 原字符串
        """
        crypt_sm4 = self.crypt_sm4
        crypt_sm4.set_key(decrypt_key.encode(), SM4_DECRYPT)
        # decrypt_value = crypt_sm4.crypt_ecb(bytes.fromhex(encrypt_value))  # bytes类型
        decrypt_value = crypt_sm4.crypt_ecb(base64.b64decode(encrypt_value.encode()))
        return self.str_to_hexStr(decrypt_value.hex())


if __name__ == '__main__':
    # str_data = {"ffffffwsdwefewd": "fefefewfwrv", "qazqaz": "vfbfrbgtrnujy"}
    key = "ADHFMUudFU1DHKHB"
    str_data = "abcd"
    SM4 = SM4()
    print("待加密内容：", str_data)
    encoding = SM4.encrypt(key, str_data)
    print("国密sm4加密后的结果：", encoding)
    print("国密sm4解密后的结果：", SM4.decrypt(key, encoding))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Copyright (C)2018 SenseDeal AI, Inc. All Rights Reserved
Author: xuwei
Email: weix@sensedeal.ai
Description:
'''

import time
import uuid
import json
import requests
from selenium import webdriver
from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities


class SpiderOp(object):
    def __init__(self, executable_path='chromedriver', headless=None,
                 ip=None, wait_time=1, try_num=2, time_to_wait_load=None,
                 downLoad=False, downLoadPath='./', js_str=None, options=None):
        """
        :param executable_path:
        :param headless:
        :param ip:
        :param wait_time:
        :param try_num:
        :param time_to_wait_load:
        :param downLoad:
        :param downLoadPath:
        :param js_str: with open('/Users/kingname/test_pyppeteer/stealth.min.js') as f: js = f.read()
        """
        if options:
            self.opt = options
        else:
            self.opt = webdriver.ChromeOptions()  # 创建Chrome参数对象
        # self.opt.headless = headless  # 把True Chrome设置成可视化无界面模式，windows/Linux 皆可
        if headless:
            self.opt.add_argument('--no-sandbox')  # 解决DevToolsActivePort文件不存在的报错
            self.opt.add_argument('window-size=1920x1080')  # 指定浏览器分辨率
            self.opt.add_argument('--disable-gpu')  # 谷歌文档提到需要加上这个属性来规避bug
            self.opt.add_argument('log-level=3')
            self.opt.add_experimental_option('excludeSwitches', ['enable-automation'])
            self.opt.add_argument('--headless')
            self.opt.add_argument(
                "--user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/79.0.3945.130 Safari/537.36")
            self.opt.add_argument("--disable-dev-shm-usage")
            self.opt.add_argument("lang=zh_CN.UTF-8")

        if downLoad:
            prefs = {'profile.default_content_settings.popups': 0, 'download.default_directory': downLoadPath}
            self.opt.add_experimental_option('prefs', prefs)


        if executable_path.startswith('http'):
            # command_executor="http://127.0.0.1:4444/wd/hub" docker selenium环境http地址
            self.driver = webdriver.Remote(
                command_executor=executable_path, desired_capabilities=DesiredCapabilities.CHROME,
                                           options=self.opt)  # 没有把Chromedriver放到python安装路径
        else:
            self.driver = webdriver.Chrome(executable_path=executable_path, options=self.opt)  # 没有把Chromedriver放到python安装路径

        if time_to_wait_load:
            self.driver.set_page_load_timeout(time_to_wait_load)
            self.driver.set_script_timeout(time_to_wait_load)

        if js_str:
            self.driver.execute_cdp_cmd("Page.addScriptToEvaluateOnNewDocument", {
                "source": js_str
            })

        self.driver_url = None
        self.wait_time = wait_time
        self.try_num = try_num
        self.current_url = ''
        self.proxy = {'https': ip}
        self.head = {
            'Host': 'emdata.pd.eastmoney.com',
            'Connection': 'keep-alive',
            'Accept-Encoding': 'gzip, deflate',
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_5) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/12.1.1 Safari/605.1.15', }

    def get_selenium_test(self, url):
        self.driver.get(url)
        time.sleep(600)

    def __send_by_xpath_list(self, send_xpath_dict, raise_error=None):
        if send_xpath_dict:
            for send_xpath in send_xpath_dict:
                try:
                    self.driver.find_element_by_xpath(send_xpath).clear()
                    self.driver.find_element_by_xpath(send_xpath).send_keys(send_xpath_dict[send_xpath])
                except Exception as e:
                    if raise_error:
                        raise e
                    # print(send_xpath)
                    # print('__send_by_xpath_list error: %s' % e)

    def __click_by_xpath_list(self, click_xpath_list, raise_error=None):
        if click_xpath_list:
            for click_xpath in click_xpath_list:
                num = 0
                while num < self.try_num:
                    try:
                        self.driver.find_element_by_xpath(click_xpath).click()
                        time.sleep(self.wait_time)

                        windows = self.driver.window_handles
                        self.current_url = self.driver.current_url
                        if len(windows) > 1:
                            self.driver.close()
                            self.driver.switch_to.window(windows[-1])
                        break
                    except Exception as e:
                        if raise_error:
                            raise e
                        num += 1
                        # print(self.driver.page_source)
                        # print(click_xpath)
                        # print('__click_by_xpath_list error: %s' % e)
                        time.sleep(self.wait_time)

    def __get_data_list_by_xpath_list(self, text_xpath_list, format='text', raise_error=None):
        xpath_data_list = []
        if text_xpath_list:
            for xpath in text_xpath_list:
                try:
                    div = self.driver.find_element_by_xpath(xpath)
                    if format == 'html':  # 用于BeautifulSoup
                        html = div.get_attribute('innerHTML')
                        xpath_data_list.append(html)
                    else:
                        xpath_data_list.append(div.text)
                except Exception as e:
                    if raise_error:
                        raise e
                    # print(xpath)
                    # print('__get_data_list_by_xpath_list error: %a' % e)
        return xpath_data_list

    def __need_appear_xpath(self, need_appear_xpath: str):
        if need_appear_xpath:
            self.driver.find_element_by_xpath(need_appear_xpath)

    def get_data_by_selenium_xpath(self, url,
                                   send_xpath_dict=None,
                                   click_xpath_list=None,
                                   need_appear_xpath=None,  # 点击之后必须存在的xpath，否则报错
                                   wait_xpath=None,
                                   text_xpath_list=None,
                                   format='text',  # html or text
                                   reload_url=False,
                                   raise_error=None,
                                   wait_time=2,
                                   mouse_pull=False,
                                   page_height_xpath=None
                                   ):
        if not self.driver_url or reload_url:
            try:
                self.driver.get(url)
            except:
                print("加载页面太慢，停止加载，继续下一步操作")
                self.driver.execute_script("window.stop()")
            self.driver_url = True

        self.__send_by_xpath_list(send_xpath_dict, raise_error=raise_error)
        self.__click_by_xpath_list(click_xpath_list, raise_error=raise_error)
        self.__need_appear_xpath(need_appear_xpath)

        if mouse_pull:
            if page_height_xpath:
                ele = self.driver.find_element_by_xpath(page_height_xpath)
                height = ele.size['height']
                loop = int(height / 1000)
            else:
                loop = 0
            # 一共下滑十次，下滑一次停顿0.5s
            for i in range(loop + 1):
                self.driver.execute_script("window.scrollBy(0, 1000)")
                time.sleep(0.5)

        self.__wait_until_xpath_appear(wait_xpath, timeout=wait_time)
        xpath_data_list = self.__get_data_list_by_xpath_list(text_xpath_list, format=format, raise_error=raise_error)
        return xpath_data_list

    def __wait_until_xpath_appear(self, wait_xpath, timeout=10):
        if wait_xpath:
            try:
                WebDriverWait(self.driver, timeout).until(EC.presence_of_element_located((By.XPATH, wait_xpath)))
                return True
            except Exception as e:
                raise e
                # return None

    def downloadBySelenium(self, url, downloadXpath: str):
        try:
            self.driver.get(url)
        except:
            print("加载页面太慢，停止加载，继续下一步操作")
            # driver.execute_script("window.stop()")
        self.driver.find_element_by_xpath(downloadXpath).click()

    def close_selenium(self):
        self.driver.close()

    def get_json_data(self, url, cookies):
        try:
            # _data = requests.get(url, headers=self.head, proxies=self.proxy)
            _data = requests.get(url, headers=self.head, cookies=cookies, verify=False)
            # print(_data)
            _data_dict = json.loads(_data.text)
            return _data_dict
        except Exception as e:
            # print(e)
            raise e

    def get_html_data(self, url):
        # _data = requests.get(url, headers=self.head, proxies=self.proxy)
        _data = requests.get(url)
        _data.encoding = 'utf-8'
        _soup = BeautifulSoup(_data.text, 'lxml')
        print(_soup)
        return _soup

    def request_download(self, url, path):
        try:
            r = requests.get(url)
            _uuid = str(uuid.uuid1())
            with open(path + _uuid + '.png', 'wb') as fp:
                fp.write(r.content)
        except Exception as e:
            print(e)

    def chunk_download(self, url, path):
        try:
            r = requests.get(url)
            _uuid = str(uuid.uuid1())
            with open(path + _uuid + '.png', 'wb') as fp:
                for chunk in r.iter_content(chunk_size=32):
                    fp.write(chunk)
        except Exception as e:
            print('chunk_download error %s' % e)


if __name__ == '__main__':
    query = '二叉树'
    url_data = SpiderOp(headless=True, wait_time=2, try_num=2)
    _url = 'https://zhihu.sogou.com/'
    url_data.get_data_by_selenium_xpath(_url,
                                        send_xpath_dict={'//*[@id="query"]': query},
                                        click_xpath_list=['//*[@id="stb"]'],
                                        need_appear_xpath='//*[@id="main"]/div[1]/p',
                                        format='html', raise_error=True
                                        )

    res_list = url_data.get_data_by_selenium_xpath(_url,
                                                   click_xpath_list=[
                                                       '//*[@id="sogou_vr_30010170_0"]',
                                                       '//*[@id="TopicMain"]/div[3]/div/div/div/div[1]/div/div[2]/div[1]',
                                                       '//*[@id="TopicMain"]/div[3]/div/div/div/div[2]/div/div[2]/div[1]',
                                                       '//*[@id="TopicMain"]/div[3]/div/div/div/div[3]/div/div[2]/div[1]'
                                                   ],
                                                   text_xpath_list=[
                                                       '//*[@id="root"]/div/main/div/div[1]/div[1]/div/div',
                                                       '//*[@id="TopicMain"]/div[3]/div/div/div/div[1]/div',
                                                       '//*[@id="TopicMain"]/div[3]/div/div/div/div[2]/div',
                                                       '//*[@id="TopicMain"]/div[3]/div/div/div/div[3]/div',
                                                   ],
                                                   format='html'
                                                   )
    url = url_data.current_url.replace('top-answers', 'hot')
    # print(url)
    discuss_list = url_data.get_data_by_selenium_xpath(url,
                                                       click_xpath_list=[
                                                           '//*[@id="TopicMain"]/div[3]/div/div/div/div[1]/div/div[2]/div[1]',
                                                           '//*[@id="TopicMain"]/div[3]/div/div/div/div[2]/div/div[2]/div[1]',
                                                           '//*[@id="TopicMain"]/div[3]/div/div/div/div[3]/div/div[2]/div[1]'
                                                       ],
                                                       text_xpath_list=[
                                                           '//*[@id="TopicMain"]/div[3]/div/div/div/div[1]/div',
                                                           '//*[@id="TopicMain"]/div[3]/div/div/div/div[2]/div',
                                                           '//*[@id="TopicMain"]/div[3]/div/div/div/div[3]/div'
                                                       ],
                                                       format='html', reload_url=True
                                                       )
    res_list = res_list + discuss_list
    print(len(res_list))
    print(res_list[0])
    url_data.close_selenium()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''                                                                                                             
Author: xuwei                                        
Email: 18810079020@163.com                                 
File: text_process.py
Date: 2021/9/16 8:46 下午
'''

import re
import jieba
import difflib
from collections import deque

stopwords = ['的', '是什么']
re_sub_regexp = '|'.join(stopwords)
re_sub = re.compile(re_sub_regexp)

re_punctuation = re.compile(
    "[！？。｡＂＃＄％＆＇（）＊＋，－／：；＜＝＞＠［＼］＾＿｀｛｜｝～｟｠｢｣､、〃《》「」『』【】〔〕〖〗〘〙〚〛〜〝〞〟〰〾〿–—‘’‛“”„‟…‧﹏.!\"#$%&\'()*+,-./:;<=>?@[\\]^_`{|}~]+")


class TextMain:

    # 移除停用词
    @classmethod
    def remove_stop_words(cls, text):
        return re_sub.sub('', text)

    @classmethod
    def remove_punctuation(cls, text):
        """
            去掉标点符号(非文本部分)
            #中文标点 ！？｡＂＃＄％＆＇（）＊＋，－／：；＜＝＞＠［＼］＾＿｀｛｜｝～｟｠｢｣､、〃》「」『』【】〔〕〖〗〘〙〚〛〜〝〞〟〰〾〿–—‘’‛“”„‟…‧﹏.
            #英文标点 !"#$%&\'()*+,-./:;<=>?@[\\]^_`{|}~
            """
        text = re_punctuation.sub('', f'{text}')
        return text

    @classmethod
    def process_text(cls, text):
        text = cls.remove_punctuation(text)
        text = cls.remove_stop_words(text)
        return text

    @classmethod
    def max_substring_length(cls, target_text, compare_text):
        """
        求两个字符串的最长公共子串的长度
        """
        seqMatch = difflib.SequenceMatcher(None, target_text, compare_text, autojunk=False)
        match = seqMatch.find_longest_match(0, len(target_text), 0, len(compare_text))
        # print(match)
        return match.size

    @classmethod
    def ratio_substring(cls, target_text, compare_text):
        """
        最长子串占目标文本长度的比率
        """
        substring_length = cls.max_substring_length(target_text, compare_text)
        return substring_length / len(target_text)

    @classmethod
    def jieba_cut(cls, sentence):
        setlast = jieba.lcut(sentence, cut_all=False)
        seg_list = [i.lower() for i in setlast if i not in stopwords]
        return " ".join(seg_list)

    @classmethod
    def public_substrings_list(cls, target_text: str, compare_text: str):
        """
        compare_text文本中含有target_text子串的列表
        :param target_text:
        :param compare_text:
        :return:
        """
        queue = deque(list(target_text))
        max_equal_str_length = 0
        max_equal_str = ''
        substring_list = []
        i = 0
        while queue:
            v = queue.popleft()
            for j in range(len(compare_text)):
                if v != compare_text[j]: continue
                equal_str = v
                for t, c in zip(target_text[i + 1:], compare_text[j + 1:]):
                    if t != c: break
                    equal_str += t
                if len(equal_str) > max_equal_str_length:
                    max_equal_str_length = len(equal_str)
                    max_equal_str = equal_str

            if max_equal_str:
                substring_list.append(max_equal_str)
                max_equal_str = ''

            i += max_equal_str_length if max_equal_str_length else 1
            for n in range(max_equal_str_length - 1):
                if not queue: break
                queue.popleft()
            max_equal_str_length = 0

        return substring_list

    @classmethod
    def similar_ratio_text_way1(cls, target_text: str, compare_text: str):
        """
        一种 compare_text 和 target_text的相似度计算方法，求 target_text 在 compare_text中的公共子串
        :param target_text:
        :param compare_text:
        :return:
        """
        substring_list = cls.public_substrings_list(target_text, compare_text)
        substring_length = sum((len(x) for x in substring_list))

        r1 = substring_length / len(target_text)
        r2 = substring_length / len(compare_text)
        if r2 > 1: r2 = 1
        ratio = (r1 + r2) / 2
        return ratio


if __name__ == '__main__':
    target_text, compare_text = "智谱华章", '北京智谱华章有限公司怎么样'
    r = TextMain.public_substrings_list(target_text, compare_text)
    print(r)

    r = TextMain.similar_ratio_text_way1(target_text, compare_text)
    print(r)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''                                                                                                             
Author: xuwei                                        
Email: 18810079020@163.com                                 
File: test_spider.py
Date: 2021/1/7 3:59 下午
'''

from xwtools import SpiderOp

if __name__ == '__main__':
    query = '二叉树'
    url_data = SpiderOp(headless=True, wait_time=2, try_num=2)
    _url = 'https://zhihu.sogou.com/'
    url_data.get_data_by_selenium_xpath(_url,
                                        send_xpath_dict={'//*[@id="query"]': query},
                                        click_xpath_list=['//*[@id="stb"]'],
                                        need_appear_xpath='//*[@id="main"]/div[1]/p',
                                        format='html', raise_error=True
                                        )

    res_list = url_data.get_data_by_selenium_xpath(_url,
                                                   click_xpath_list=[
                                                       '//*[@id="sogou_vr_30010170_0"]',
                                                       '//*[@id="TopicMain"]/div[3]/div/div/div/div[1]/div/div[2]/div[1]',
                                                       '//*[@id="TopicMain"]/div[3]/div/div/div/div[2]/div/div[2]/div[1]',
                                                       '//*[@id="TopicMain"]/div[3]/div/div/div/div[3]/div/div[2]/div[1]'
                                                   ],
                                                   text_xpath_list=[
                                                       '//*[@id="root"]/div/main/div/div[1]/div[1]/div/div',
                                                       '//*[@id="TopicMain"]/div[3]/div/div/div/div[1]/div',
                                                       '//*[@id="TopicMain"]/div[3]/div/div/div/div[2]/div',
                                                       '//*[@id="TopicMain"]/div[3]/div/div/div/div[3]/div',
                                                   ],
                                                   format='html'
                                                   )
    url = url_data.current_url.replace('top-answers', 'hot')
    # print(url)
    discuss_list = url_data.get_data_by_selenium_xpath(url,
                                                       click_xpath_list=[
                                                           '//*[@id="TopicMain"]/div[3]/div/div/div/div[1]/div/div[2]/div[1]',
                                                           '//*[@id="TopicMain"]/div[3]/div/div/div/div[2]/div/div[2]/div[1]',
                                                           '//*[@id="TopicMain"]/div[3]/div/div/div/div[3]/div/div[2]/div[1]'
                                                       ],
                                                       text_xpath_list=[
                                                           '//*[@id="TopicMain"]/div[3]/div/div/div/div[1]/div',
                                                           '//*[@id="TopicMain"]/div[3]/div/div/div/div[2]/div',
                                                           '//*[@id="TopicMain"]/div[3]/div/div/div/div[3]/div'
                                                       ],
                                                       format='html', reload_url=True
                                                       )
    res_list = res_list + discuss_list
    print(len(res_list))
    print(res_list[0])
    url_data.close_selenium()


from .config_log import config, log, init_log, catch_error_log  # 配置文档读取和log日志以及捕获异常装饰器

from .cut_word import TextEtl  # 文本处理

from .utils import get_url_uuid, drop_element_contain_in_str_list, load_json, dump_json, get_quote  # 常用工具

from .email import EmailFactory, send_mail  # 发邮件
from .wechat import WechatFactory  # 发微信

from .etl_time import TimeEtl, now_time, TimeCost, format_time_cost, make_timestamp, sleep  # 时间相关

from .ReParse import Parse  # 正则相关

from .run_on_time import RunOnTime  # 指定时间运行程序

from .queue_main import RabbitProducer, RabbitConsumer  # 消息队列，生产者和消费者
from .queue2 import RabbitConsumer2  # 消息队列，消费者

# 数据库操作相关
from .es import BaseSearcher, BaseIndexer, getNerByEs  # es model操作
from .elasticsearch_op import EsOp, get_es_client  # es直接操作

from .redis_operate import RedisOp, get_redis_client  # redis操作

from .mysql_op import UpdateTable, MysqlOp  # mysql操作
from .db_op import DB  # mysql操作，实现了mysql连接池，长链接用

from .sqlserver_op import SqlServerOp  # sqlserver操作

from .mongo_op import MongoOp  # mongo操作

from .hive_op import HiveOp  # hive操作

# from .neo4j_op import Neo4jOp # neo4j操作

from .plt_graph import series_plot_graph  # pandas的series画图

from .spider_op import SpiderOp  # 爬虫selenium

# 多进程和线程相关
from .multi_process_thread import multi_process_map, multi_thread_map, multi_thread_return, \
    multi_thread_return_funs_list

from .url_uuid import get_uuid5, get_md5

# uuid相关
# from .url_uuid import get_url_uuid, get_url_uuid_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''                                                                                                             
Author: xuwei                                        
Email: 18810079020@163.com                                 
File: SM2.py
Date: 2021/5/28 1:50 下午
'''

"""
1- 国产非对称加密算法，后端生成公钥给前端，前端通过公钥加密后反给后端，后端通过私钥解密；
2- 不可逆加密密码和验证密码是否正确
"""

from gmssl import sm2, func


# (pysmx) pip install snowland-smx==0.3.2a1
def generate_keys():
    from pysmx.SM2 import generate_keypair
    pk, sk = generate_keypair()
    print(f'public_key: {pk.hex()}')
    print(f'private_key: {sk.hex()}')
    return pk.hex(), sk.hex()


class EncryptBySM2(object):
    def __init__(self, public_key, private_key):
        self.__sm2_crypt = sm2.CryptSM2(public_key=public_key, private_key=private_key)

    def encrypt(self, content: str):
        # 16进制的公钥和私钥
        enc_data = self.__sm2_crypt.encrypt(content.encode()).hex()
        return enc_data

    def decrypt(self, enc_data: str):
        dec_data = self.__sm2_crypt.decrypt(bytes.fromhex(enc_data)).decode()
        return dec_data

    def encrypt_password(self, password: str):
        salt = func.random_hex(self.__sm2_crypt.para_len)
        enc_password = self.__sm2_crypt.sign(password.encode(), salt)  # 16进制
        return enc_password

    def ver_password(self, password: str, enc_password: str):
        ver = self.__sm2_crypt.verify(enc_password, password.encode())  # 16进制
        return ver


if __name__ == '__main__':
    private_key = 'ca28c0f5bb4588df803b6ccbd030026f0d395384bc4d68783099044234cafaac'
    public_key = 'bdf27a47becf268fcde4336d965f23fc8c14039d1fe89144d98de23f01873899442e5ef956fe1a5a7c3fe32e5c84ebe9867f6a92f48255a0d86ce4b72252a624'

    # 生成公钥， 私钥
    # public_key,private_key = generate_keys()

    encSM2 = EncryptBySM2(public_key=public_key, private_key=private_key)

    # content = "这是一个测试"
    # enc_data = encSM2.encrypt(content)
    # print(enc_data)
    # dec_data = encSM2.decrypt(enc_data)
    # print(dec_data)

    password = "xuwei2020"
    enc_password = encSM2.encrypt_password(password)
    print(enc_password)
    ver = encSM2.ver_password(password, enc_password)
    print(ver)



import json
import re
import shortuuid
import numpy
import datetime
from datetime import date
from urllib.parse import quote

'''
不依赖本项目的其它包，全是线上包
'''


class MyException(Exception):  # 让MyException类继承Exception
    def __init__(self, str_describe):
        self.msg = str_describe


##datetime.datetime is not JSON serializable 报错问题解决
class CJsonEncoder(json.JSONEncoder):
    def default(self, obj):
        if obj != obj:
            return None
        elif isinstance(obj, datetime.datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, date):
            return obj.strftime("%Y-%m-%d")
        elif isinstance(obj, numpy.integer):
            return int(obj)
        elif isinstance(obj, numpy.floating):
            return float(obj)
        elif isinstance(obj, numpy.ndarray):
            return obj.tolist()
        elif isinstance(obj, bytes):
            return str(obj)
        return json.JSONEncoder.default(self, obj)


# URL 编码
def get_quote(text):
    return quote(text)


def dump_json(data):
    return json.dumps(data, ensure_ascii=False, cls=CJsonEncoder)


def load_json(data):
    try:
        return json.loads(data)
    except:
        return None


def format_blank(content):
    clear_list = {
        u'\u2002': '', u'\u2003': '', u'\u2009': '', u'\u200c\u200d': '', u'\xa0': '', '&nbsp;': '',
        '&ensp;': '', '&emsp;': '', '&zwj;': '', '&zwnj;': ''
    }
    rep = dict((re.escape(k), v) for k, v in clear_list.items())
    pattern = re.compile('|'.join(rep.keys()))
    content = pattern.sub(lambda x: rep[re.escape(x.group(0))], content)
    return content


def get_url_uuid(url):
    return shortuuid.uuid(name=url)


# ['总论', '自然', '自然辩证法', '自然辩证法总论'],
def drop_element_contain_in_str_list(str_list, reverse=True, case_sensitive=False):
    """
    :param str_list: 列表元素必须全是字符串
    :param reverse: reverse=True代表按字符串长度降序排列
    :param case_sensitive: 大小写是否敏感，默认不区分大小写
    :return:
    """
    remove_elements = set()
    all_elements = set(str_list)

    group_dict = dict()
    for obj in all_elements:
        group_dict[len(obj)] = group_dict.get(len(obj), []) + [obj]

    name_group_list_tuple = sorted(group_dict.items(), key=lambda x: x, reverse=True)

    for i in range(1, len(name_group_list_tuple)):
        layer_elements_list = name_group_list_tuple[i][1]
        for elements_tuple in name_group_list_tuple[:i]:
            current_str = '、'.join(elements_tuple[1])
            for ele in layer_elements_list:
                if not case_sensitive:
                    if ele.lower() in current_str.lower():
                        remove_elements.add(ele)
                else:
                    if ele in current_str:
                        remove_elements.add(ele)

    remain_elements = all_elements - remove_elements
    if reverse:
        remain_elements = sorted(remain_elements, key=lambda x: len(x), reverse=reverse)
    return list(remain_elements)



if __name__ == '__main__':
    str_list = ['总论', '自然xw', '自然XW辩证法', '自然辩证法总论']
    r = drop_element_contain_in_str_list(str_list, case_sensitive=True)
    print(r)
    print(str_list)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Copyright (C)2018 SenseDeal AI, Inc. All Rights Reserved
File: .py
Author: xuwei
Email: weix@sensedeal.ai
Last modified: date
Description: 对原始文本进行处理，分词后保存txt，供word2vec训练词向量使用。
'''


import jieba
import re

class TextEtl(object):

    def __init__(self, stopWordPath = None):
        self.stopwords = None
        if stopWordPath:
            self.stopwords = [line.strip() for line in open(stopWordPath, 'r', encoding='utf-8').readlines()]

    def numLower(self, text):
        # constants for chinese_to_arabic
        cn_num = {
            '〇': 0, '一': 1, '二': 2, '三': 3, '四': 4, '五': 5, '六': 6, '七': 7, '八': 8, '九': 9, '零': 0,
            '壹': 1, '贰': 2, '叁': 3, '肆': 4, '伍': 5, '陆': 6, '柒': 7, '捌': 8, '玖': 9, '貮': 2, '两': 2,
            '仟': '000', }
        for _key in cn_num:
            text = text.replace(_key, str(cn_num[_key]))
        return text

    def filter_emoji(self, line):
        """
        过滤表情
        """
        try:
            # python UCS-4 build的处理方式
            highpoints = re.compile(u'[\U00010000-\U0010ffff]')
        except re.error:
            # python UCS-2 build的处理方式
            highpoints = re.compile(u'[\uD800-\uDBFF][\uDC00-\uDFFF]')

        return highpoints.sub(u'', line)

    def remove_other(self, line: str):
        """
        去除txt文本中的空格、数字、特定字母等
        #去掉文本行里面的空格、换行\t、数字（其他有要去除的也可以放到' \t1234567890'里面）
        """
        lines = filter(lambda ch: ch not in ' \t1234567890☝✌ ☺', line)
        return ''.join(lines)

    def remove_punctuation(self, line):
        """
        去掉标点符号(非文本部分)
        #中文标点 ！？｡＂＃＄％＆＇（）＊＋，－／：；＜＝＞＠［＼］＾＿｀｛｜｝～｟｠｢｣､、〃》「」『』【】〔〕〖〗〘〙〚〛〜〝〞〟〰〾〿–—‘’‛“”„‟…‧﹏.
        #英文标点 !"#$%&\'()*+,-./:;<=>?@[\\]^_`{|}~
        """
        try:
            line = re.sub(
                "[！？。｡＂＃＄％＆＇（）＊＋，－／：；＜＝＞＠［＼］＾＿｀｛｜｝～｟｠｢｣､、〃《》「」『』【】〔〕〖〗〘〙〚〛〜〝〞〟〰〾〿–—‘’‛“”„‟…‧﹏.!\"#$%&\'()*+,-./:;<=>?@[\\]^_`{|}~]+",
                "", line)
        except Exception as e:
            print(e)
        return line

    def __jieba_cut_filter(self, text):
        text = text.encode("utf-8")
        word_list = jieba.cut(text)
        word_list = list(filter(lambda x: len(x) > 1, word_list))  # 去掉长度小于1的词
        return word_list

    def _remove_stopwords(self, word_list):
        """
        创建停用词list很专业，这里也可以过滤标点符号，替代函数def remove_punctuation(line)，
        参数line_cutted是分词后的列表。
        """
        if self.stopwords:
            word_list = list(filter(lambda x: x not in self.stopwords, word_list))
        return word_list

    def getWordListByCut(self, sentence):
        """
        对一行句子进行分词，去数字、空格、换行和标点符合等，并丢弃停用词，可以实现Series的并行处理
        """
        #jieba添加词典
        #     jieba.add_word('非负面')
        text = sentence.replace("\n", "")
        text = text.replace('\\r', '')
        text = text.replace('\\n', '')
        text = text.replace('\r', '')
        text = text.replace('　　', '。')
        text = text.replace(' ', '')

        text = self.filter_emoji(text)
        text = self.remove_other(text)
        text = self.remove_punctuation(text)
        word_list = self.__jieba_cut_filter(text)
        word_list = self._remove_stopwords(word_list)  #分词后的列表去除停用词
        return word_list


if __name__ == '__main__':
    sentence = '　　墨斐、孙晓东加盟后，观致汽车正在迅速组建“通用队”。3月16日，观致员工收到的内部邮件显示：来自上海通用的徐宛将担任观致汽车市场与销售部公关总监。而就在不到一周之前的3月12日，也就是原观致公关总监梁虹离职的同一天，观致的员工收到由孙晓东亲自签发的信函，凌海加盟观致汽车，担任市场与销售部网络总监。\r\n　　徐宛和凌海均来自“通用系”，徐最早曾任职上海通用公关部，加盟观致前任职雪佛兰品牌部；而凌是第一任雪佛兰品牌总监，此后加盟吉利汽车，担任吉利汽车副总经理。\r\n　　从孙晓东加盟观致到墨斐加盟，观致的营销岗位纷纷开始由“通用系”接手，而上海通用是业内公认的“营销之王”，这也被视为墨斐要为观致下一剂营销“猛药”的开端。\r\n　　“通用队”雏形\r\n　　3月12日孙晓东签发的邮件显示：凌海的主要工作是负责中国市场零售网络的扩张和优化。随着凌海和徐宛的到来，观致组建起了CEO墨斐、 市场及销售执行总监孙晓东为首的“通用队”雏形，而这个通用队，是要来解决观致的实际问题的。\r\n　　“观致目前要做的第一件事是要树立品牌定位，提高品牌的知名度。”2月2日墨斐上任观致CEO的七个小时后就曾表示。而根据董事会给他的目标：观致的目标是要让中国的企业，可以在国际市场竞争，可以与其他国际品牌媲美。\r\n　　不过，没有销量作为基础，一切都是空谈。“孙总来了以后，提出第一步今年3月销量破千的目标。”观致知情人士透露，而之前，观致的月销量仅百辆级别。而销售网络的能力，是决定销量很重要的因素，之前，观致也一直受困于网络发展速度太慢。\r\n　　近年来，随着市场进入平稳期，有实力的经销商纷纷转向豪华车，包括一汽丰田、斯柯达等合资企业，都不断有经销商退出，这对于全新的观致，是致命的要害。观致的网络发展一直被业内垢病，至今为止，观致的网络远未达到目标，去年接受记者采访时，时任观致CEO郭谦和市场及销售执行总监卫思梵纷纷表示了对网络发展的不满意。\r\n　　而孙晓东的这份邮件表示：凌海以前的工作经验，将使其很胜任这份工作，帮助观致达到2015年的目标。\r\n　　大众制造、通用营销\r\n　　上海通用的营销能力是有口皆碑的，“通用队”组建后，推动在营销层面尽快建立起通用的流程和体系并不难。虽然处于这样高压的环境，但墨斐并未对观致的设计团队进行调整。以何歌特带领的观致设计团队并无变化，而产品也按照之前的既定计划在推进。在产品上，观致仍然会延续之前的高标准，仍然以国际化标准对标大众。\r\n　　墨斐本身虽然不是中国人，但绝对是中国通，他曾经为了与上汽之间更畅通的合作，而与时任上汽董事长胡茂元坐在一个办公室办公。\r\n　　上任观致第一天，记者连线墨斐，他便在电话中告诉记者，观致面前最缺的是品牌。品牌的打造最关键是要了解消费者需求。\r\n　　郭谦带领的第一代观致管理层，将观致对标于大众，郭谦多次表示，观致是“大众品质，现代价格”。观致的确做到了大众品质。\r\n　　“观致从产品上看已经实现了国际标准，在国际市场也有竞争力，所以我对达成目标非常有信心。”墨斐也承认。不过观致并没有做到现代价格，原因很简单，汽车是一个以规模论成本的产业，同样的品质，规模越大，成本越低。\r\n　　虽然与大众相比，观致不需要向总部缴纳技术转让费，但从无到有打造一个平台，为之付出的成本也非常高昂。此前，有消息称观致的投资已经超过100亿人民币。如果销量能迅速打开，100亿人民币的投资额并不算大，一旦销量不能达到预期目标，“节流”成为第二代管理层不得不面对的现实问题。\r\n　　人才本土化\r\n　　“当时我在大众，本来非常安稳，为了梦想才来到观致。”梁虹在离职后的第一时间这样对记者说。她说她并不后悔当初的选择。与任何一个前期创业者一样，处于起步期的观致，需要高强度的投入，每个人都为它付出了沉重的代价。\r\n　　实际上，在梁之前，质量部的执行总监Ralf已经离开观致，制造部的执行总监和网络部总监也已离职，除了主动离开的梁虹，其中也不乏有“被离职”的。据观致的内部员工透露，墨斐到观致以后，将裁员列入计划。并提出了外籍员工减少50位，中方籍员工减少200位的裁员目标。\r\n　　记者了解到，墨斐的裁员计划已经被分解到各个部门，不过，由于观致主动离职的也不在少数，所以，这一裁员计划给内部员工造成的压力并不大。但是，对于外籍员工还是有一定压力的。\r\n　　“同样的职位，外籍员工的收入远远高于本土员工，这是行业的惯例。”观致员工透露。对于一家实际销量远远低于预期销量，连续处于亏损状态的企业而言，墨斐上任后第一时间裁员，就是想改善观致的盈利状态。而有消息称，之前因为缺乏了解本土市场的人才，观致大量启用高成本的外籍顾问和咨询公司，而大量启用本土人才后，墨斐也可能会减少这部分投入。\r\n　　人才本土化是美系企业区别于欧系、日系乃至韩系企业很重要的一个因素，美系企业一般大胆启用本土人才，以上海通用为例子，除了核心部门以外，基本都是本土化人才。“观致一些部门执行总监离职后，该职位尚在空缺期，很可能未来都会启用本土人才填补空缺。”知情人士透露，启用本土人才不仅仅成本相对更低，且本土人才更了解本土市场，更接地气，这对观致而言，可谓一举两得。\r\n】【、|'
    rs = TextEtl('./stop_word.txt').getWordListByCut(sentence)
    print(rs)



import base64
import hashlib

'''
采用AES对称加密算法
pip3 install -i https://pypi.douban.com/simple pycryptodome
'''

keyStr = "sdf8g4edxuweif80"


class Encryption(object):

    @classmethod
    def encrypt_key(cls, text):
        from Crypto.Cipher import AES
        private_key = hashlib.sha256(keyStr.encode()).digest()
        rem = len(text) % 16
        padded = str.encode(text) + (b'\0' * (16 - rem)) if rem > 0 else str.encode(text)
        iv = bytes([0] * 16)
        cipher = AES.new(private_key, AES.MODE_CFB, iv, segment_size=128)
        enc = cipher.encrypt(padded)[:len(text)]
        return base64.b64encode(iv + enc).decode()

    @classmethod
    def decrypt_key(cls, text):
        from Crypto.Cipher import AES
        private_key = hashlib.sha256(keyStr.encode()).digest()
        text = base64.b64decode(text)
        iv, value = text[:16], text[16:]
        rem = len(value) % 16
        padded = value + (b'\0' * (16 - rem)) if rem > 0 else value
        cipher = AES.new(private_key, AES.MODE_CFB, iv, segment_size=128)
        return (cipher.decrypt(padded)[:len(value)]).decode()

    @classmethod
    def encryt_dict(cls, config_map, ignore=None):
        # ignore 为忽略的字段，默认值是none，值为list，如['a', 'b']
        _res = {}
        for _k, _v in config_map.items():
            if ignore and _k in ignore:
                _res[_k] = _v
                continue
            _res[_k] = cls.encrypt_key(_v)
        return _res

    @classmethod
    def decrypt_dict(cls, config_map, ignore=None):
        # ignore 为忽略的字段，默认值是none，值为list，如['a', 'b']
        _res = {}
        for _k, _v in config_map.items():
            if ignore and _k in ignore:
                _res[_k] = _v
                continue
            try:
                _res[_k] = cls.decrypt_key(_v)
            except:
                _res[_k] = _v
        return _res


import bcrypt


def encrypt_by_bcrypt(password: str):
    # 用一个随机的盐值来加密密码
    # bcrypt.gensalt() 还可以接受一个参数来控制它要计算多少次，默认是 12
    hashed = bcrypt.hashpw(password.encode(), bcrypt.gensalt())
    # 下面这个 hashed 就是加密后的值
    return hashed.decode()


def verify_password_by_bcrypt(password: str, encrypt_password: str):
    # 验证明文密码是不是加密后的值
    return bcrypt.checkpw(password.encode(), encrypt_password.encode())


if __name__ == '__main__':
    r = Encryption.encrypt_key("150204200112110016")
    print(r == "AAAAAAAAAAAAAAAAAAAAADvOo1yAl/RC4cAvHim6QOQqCA==")
    print(Encryption.decrypt_key(r))

    password = '150204200112110016'
    encrypt_password = encrypt_by_bcrypt(password)
    print(encrypt_password)
    print(verify_password_by_bcrypt(password, encrypt_password))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Copyright (C)2018 SenseDeal AI, Inc. All Rights Reserved
File: .py
Author: xuwei
Email: weix@sensedeal.ai
Last modified: date
Description:
'''

import datetime
import time


class TimeEtl(object):

    @classmethod
    def ge_now(cls):
        return datetime.datetime.now()

    @classmethod
    def time_str(cls, _now, format='%Y-%m-%d %H:%M:%S'):
        _str_now = _now.strftime(format)
        return _str_now

    @classmethod
    def time_stamp(cls, _now):
        un_time = time.mktime(_now.timetuple())
        return int(un_time)

    @classmethod
    def str_time(cls, _str, format='%Y-%m-%d %H:%M:%S'):
        _time = datetime.datetime.strptime(_str, format)
        return _time

    @classmethod
    def str_time_stamp(cls, _str):
        _time = cls.str_time(_str)
        _time_stamp = cls.time_stamp(_time)
        return _time_stamp

    @classmethod
    def stamp_time(cls, _stamp, format="%Y-%m-%d %H:%M:%S"):
        _dt = time.strftime(format, time.localtime(_stamp))
        return _dt

    @classmethod
    def before_day(cls, _now, _day, format="%Y-%m-%d %H:%M:%S"):
        _time = (_now - datetime.timedelta(days=_day)).strftime(format)
        return _time

    @classmethod
    def after_day(cls, _now, _day, format="%Y-%m-%d %H:%M:%S"):
        _time = (_now + datetime.timedelta(days=_day)).strftime(format)
        return _time


def format_time_cost(val):
    result = ''
    if val >= 3600:
        hour = int(val / 3600)
        result += str(hour) + '小时'
        val -= hour * 3600
    if val >= 60:
        min = int(val / 60)
        result += str(min) + '分'
        val -= min * 60
    result += str(val) + '秒'
    return result


def now_time(format='%Y-%m-%d %H:%M:%S'):
    # 获取当前时间str
    now = datetime.datetime.now().strftime(format)
    return now


def make_timestamp(time_str=None, format='%Y-%m-%d'):
    if not time_str or time_str == 'NaT':
        time_str = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        format = '%Y-%m-%d %H:%M:%S'
    d = datetime.datetime.strptime(time_str, format)
    t = d.timetuple()
    _timestamp = int(time.mktime(t))
    return _timestamp


def sleep(seconds):
    time.sleep(seconds)


def get_current_second():
    return int(round(time.time()))


def get_current_millisecond():
    return int(round(time.time() * 1000))


class TimeCost(object):
    __start = get_current_millisecond()

    @classmethod
    def show_time_diff(cls):
        return format_time_cost((get_current_millisecond() - cls.__start) / 1000)

    @classmethod
    def reset_time(cls):
        cls.__start = get_current_millisecond()

    @classmethod
    def time_diff(cls):
        return get_current_millisecond() - cls.__start


if __name__ == '__main__':
    _now = TimeEtl.ge_now()
    # _str_time = TimeEtl.time_stamp(_now)
    # print(_str_time)
    #
    _time = TimeEtl.before_day(_now, 5)
    print(_time)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''                                                                                                             
Author: xuwei                                        
Email: 18810079020@163.com                                 
File: plt_graph.py
Date: 2020/7/6 4:10 下午
'''



import matplotlib.pyplot as plt

def series_plot_graph(series, title, file_path='.', xlabel='', ylabel='', width=8, height=7, fontsize=9, **kwargs):
    plt.figure(figsize=(width, height))
    ax = series.plot(kind='bar', fontsize=fontsize, **kwargs)
    plt.title(title, fontsize=fontsize)
    plt.xlabel(xlabel, fontsize=fontsize)
    plt.ylabel(ylabel, fontsize=fontsize)
    times_list = series.to_list()
    for i in range(len(times_list)):
        plt.text(i, times_list[i], times_list[i], ha='center', va='bottom',
                 fontdict={'size': fontsize, 'color': 'black'})
    plt.tight_layout() # 解决ticklabel或者title等过长、或者过大的话，显示不全的问题
    plt.show()
    fig = ax.get_figure()
    fig.savefig('%s/%s.png' % (file_path, title))


import os
from setuptools import setup

# 根据es版本安装python es库
# pymongo
# "elasticsearch-dsl>=6.3.1",
# "elasticsearch-dsl>=7.2.0",

requirements = [
    'pika',
    'numpy',
    'shortuuid',
    'threadpool',
    'redis',
    'elasticsearch_dsl',
    'matplotlib',
    'jieba',
    'wheel',
    'PyMySQL',
    'pypinyin',
    'selenium',
    'elasticsearch',
    'pymongo',
    'pandas',
    'requests',
    'beautifulsoup4',
    'sqlalchemy'
]

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='xwtools',
    version='2.0.6',
    packages=[
        "xwtools",
    ],
    license='BSD License',  # example license
    description='xwtools',
    long_description='这是一个通用的python工具包，帮助你快速的开发项目',
    install_requires=requirements,
    long_description_content_type="text/markdown",
    url='https://github.com/xulehexuwei',
    author='xuwei',
    author_email='15200813194@163.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.1',  # replace "X.Y" as appropriate
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Copyright (C)2018 SenseDeal AI, Inc. All Rights Reserved
Author: xuwei
Email: weix@sensedeal.ai
Description:
'''
# py2neo == 2021
from .config_log import config
from py2neo import Graph, Node, Relationship, NodeMatcher


class Neo4jOp():
    def __init__(self, host='', port='', user='', password='', label=''):
        if label:
            host = config(label, 'host')
            port = config(label, 'port')
            user = config(label, 'user')
            password = config(label, 'pass')
        self._graph = Graph('bolt://' + host + ':' + port, auth=(user, password))
        self._node_map = dict()
        self._edge_map = dict()
        self._node_relation_map = dict()
        self._node_edges = dict()

    def get_relationship_types(self):
        r = self._graph.schema.relationship_types
        return r

    def get_node_labels(self):
        r = self._graph.schema.node_labels
        return r

    def merge_node(self, label, unique_key, param_dict):
        tx = self._graph.begin()
        tx.merge(Node(label, **param_dict), label, unique_key)
        tx.commit()

    def create(self, node):
        self._graph.create(node)

    def push_ogm(self, node):
        self._graph.push(node)

    def delete_by_label(self, label):
        self._graph.run("MATCH (n:%s) delete n" % label)

    def truncate_neo4j(self):
        self._graph.delete_all()

    def run_cql(self, cql):
        return self._graph.run(cql)

    # 有返回Node，无返回None，返回None的话调用Node创建节点
    def nodes_match(self, labels, **kwargs):
        _node_obj = self._graph.nodes.match(labels, **kwargs).first()
        return _node_obj

    def nodes_query(self, labels, **kwargs):
        _nodes_obj = self._graph.nodes.match(labels, **kwargs)
        return _nodes_obj

    def relations_query(self, node, r_type):
        data = self._graph.relationships.match(node, r_type)
        return data

    def query_node_by_label(self, label):
        _list_obj = list(self._graph.nodes.match(label))
        _list_dict = [dict(_obj) for _obj in _list_obj]
        return _list_dict


if __name__ == '__main__':
    op_neo4j = Neo4jOp(label='neo4j')
    r = op_neo4j.get_node_labels()
    print(r)
    r = op_neo4j.get_relationship_types()
    print(r)
    r = op_neo4j.nodes_match('Company', code='10085631')
    print(dict(r))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''                                                                                                             
Author: xuwei                                        
Email: 18810079020@163.com                                 
File: strQB.py
Date: 2021/4/14 3:53 下午
'''


def strQ2B(ustring):
    """把字符串全角转半角"""
    ss = []
    for s in ustring:
        rstring = ""
        for uchar in s:
            inside_code = ord(uchar)
            if inside_code == 12288:  # 全角空格直接转换
                inside_code = 32
            elif (inside_code >= 65281 and inside_code <= 65374):  # 全角字符（除空格）根据关系转化
                inside_code -= 65248
            rstring += chr(inside_code)
        ss.append(rstring)
    return ''.join(ss)


def strB2Q(ustring):
    ss = []
    for s in ustring:
        rstring = ""
        for uchar in s:
            inside_code = ord(uchar)
            if inside_code == 32:  # 全角空格直接转换
                inside_code = 12288
            elif (inside_code >= 33 and inside_code <= 126):  # 全角字符（除空格）根据关系转化
                inside_code += 65248
            rstring += chr(inside_code)
        ss.append(rstring)
    return ''.join(ss)


if __name__ == '__main__':
    r = strQ2B("２２０７２２２０００１０２５０８１７")
    print(r)

    print(strB2Q(r))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Author: xuwei
Email: 18810079020@163.com
File: chinese2digits.py
Date: 2021/4/7 11:54 上午
'''
import re
from typing import Union
import time
import pathlib
import logging

# 把汉语句子中的汉字（大小写）数字转为阿拉伯数字，不能识别“百分之”
common_used_numerals = {'零': 0, '一': 1, '二': 2, '两': 2, '三': 3, '四': 4, '五': 5, '六': 6, '七': 7, '八': 8,
                        '九': 9, '十': 10, u'〇': 0, u'壹': 1, u'贰': 2, u'叁': 3, u'肆': 4, u'伍': 5, u'陆': 6,
                        u'柒': 7, u'捌': 8, u'玖': 9, '拾': 10, '百': 100, '千': 1000, u'貮': 2, u'俩': 2, '佰': 100,
                        '仟': 1000, '萬': 10000, '万': 10000, '亿': 100000000, '億': 100000000, '兆': 1000000000000}


def chinese2digits(uchars_chinese):
    total = 0
    r = 1  # 表示单位：个十百千...
    for i in range(len(uchars_chinese) - 1, -1, -1):
        val = common_used_numerals.get(uchars_chinese[i])
        if val >= 10 and i == 0:  # 应对 十三 十四 十*之类
            if val > r:
                r = val
                total = total + val
            else:
                r = r * val
                # total =total + r * x
        elif val >= 10:
            if val > r:
                r = val
            else:
                r = r * val
        else:
            total = total + r * val
    return total


num_str_start_symbol = ['一', '二', '两', '三', '四', '五', '六', '七', '八', '九', '十',
                        '壹', '贰', '叁', '肆', '伍', '陆', '柒', '捌', '玖', '拾', '貮', '俩', ]
more_num_str_symbol = ['零', '一', '二', '两', '三', '四', '五', '六', '七', '八', '九', '十', '百', '千', '万', '亿',
                       '〇', '壹', '贰', '叁', '肆', '伍', '陆', '柒', '捌', '玖', '拾', '貮', '俩', '佰', '仟', '萬', '億', '兆']


def ChineseNumToArab(oriStr):
    lenStr = len(oriStr)
    aProStr = ''
    if lenStr == 0:
        return aProStr
    hasNumStart = False
    numberStr = ''
    for idx in range(lenStr):
        if oriStr[idx] in num_str_start_symbol:
            if not hasNumStart:
                hasNumStart = True
            numberStr += oriStr[idx]
        else:
            if hasNumStart:
                if oriStr[idx] in more_num_str_symbol:
                    numberStr += oriStr[idx]
                    continue
                else:
                    numResult = str(chinese2digits(numberStr))
                    numberStr = ''
                    hasNumStart = False
                    aProStr += numResult
            aProStr += oriStr[idx]
            pass
    if len(numberStr) > 0:
        resultNum = chinese2digits(numberStr)
        aProStr += str(resultNum)
    return aProStr


def get_default_conf() -> dict:
    config_data = {
        'number_cn2an': {'零': 0, '一': 1, '壹': 1, '幺': 1, '二': 2, '贰': 2, '两': 2, '三': 3, '叁': 3, '四': 4, '肆': 4, '五': 5,
                         '伍': 5, '六': 6, '陆': 6, '七': 7, '柒': 7, '八': 8, '捌': 8, '九': 9, '玖': 9},
        'unit_cn2an': {'十': 10, '拾': 10, '百': 100, '佰': 100, '千': 1000, '仟': 1000, '万': 10000, '亿': 100000000},
        'unit_low_an2cn': {10: '十', 100: '百', 1000: '千', 10000: '万', 100000000: '亿'},
        'number_low_an2cn': {0: '零', 1: '一', 2: '二', 3: '三', 4: '四', 5: '五', 6: '六', 7: '七', 8: '八', 9: '九'},
        'number_up_an2cn': {0: '零', 1: '壹', 2: '贰', 3: '叁', 4: '肆', 5: '伍', 6: '陆', 7: '柒', 8: '捌', 9: '玖'},
        'unit_low_order_an2cn': ['', '十', '百', '千', '万', '十', '百', '千', '亿', '十', '百', '千', '万', '十', '百', '千'],
        'unit_up_order_an2cn': ['', '拾', '佰', '仟', '万', '拾', '佰', '仟', '亿', '拾', '佰', '仟', '万', '拾', '佰', '仟'],
        'strict_cn_number': {'零': '零', '一': '一壹', '二': '二贰', '三': '三叁', '四': '四肆', '五': '五伍', '六': '六陆', '七': '七柒',
                             '八': '八捌', '九': '九玖', '十': '十拾', '百': '百佰', '千': '千仟', '万': '万', '亿': '亿'},
        'normal_cn_number': {'零': '零', '一': '一壹幺', '二': '二贰两', '三': '三叁仨', '四': '四肆', '五': '五伍', '六': '六陆', '七': '七柒',
                             '八': '八捌', '九': '九玖', '十': '十拾', '百': '百佰', '千': '千仟', '万': '万', '亿': '亿'}}
    return config_data


def get_logger(name: str = "cn2an", level: str = "info") -> logging.Logger:
    # set logger
    logger = logging.getLogger(name)

    level_dict = {
        "debug": logging.DEBUG,
        "info": logging.INFO,
        "warning": logging.WARNING,
        "error": logging.ERROR,
        "critical": logging.CRITICAL
    }
    logger.setLevel(level_dict[level])

    if not logger.handlers:
        # file handler
        log_path = log_path_util()
        fh = logging.FileHandler(log_path)
        fh.setLevel(logging.INFO)
        fh_fmt = logging.Formatter("%(asctime)-15s %(filename)s %(levelname)s %(lineno)d: %(message)s")
        fh.setFormatter(fh_fmt)

        # stream handler
        console = logging.StreamHandler()
        console.setLevel(logging.DEBUG)
        console_fmt = logging.Formatter("%(filename)s %(levelname)s %(lineno)d: %(message)s")
        console.setFormatter(console_fmt)

        logger.addHandler(fh)
        logger.addHandler(console)

    return logger


def log_path_util(name: str = "cn2an") -> str:
    day = time.strftime("%Y-%m-%d", time.localtime())
    log_path = pathlib.Path(f"./log/{day}")
    if not log_path.exists():
        log_path.mkdir(parents=True)
    return f"{str(log_path)}/{name}.log"


class Cn2An(object):
    def __init__(self) -> None:
        self.conf = get_default_conf()
        self.all_num = "".join(list(self.conf["number_cn2an"].keys()))
        self.all_unit = "".join(list(self.conf["unit_cn2an"].keys()))
        self.strict_cn_number = self.conf["strict_cn_number"]
        self.normal_cn_number = self.conf["normal_cn_number"]
        self.check_key_dict = {
            "strict": "".join(self.strict_cn_number.values()) + "点负",
            "normal": "".join(self.normal_cn_number.values()) + "点负",
            "smart": "".join(self.normal_cn_number.values()) + "点负" + "01234567890.-"
        }
        self.pattern_dict = self.__get_pattern()
        self.ac = An2Cn()

    def cn2an(self, inputs: str = None, mode: str = "strict") -> int:
        if inputs is not None or inputs == "":
            # 检查转换模式是否有效
            if mode not in ["strict", "normal", "smart"]:
                raise ValueError("mode 仅支持 strict normal smart 三种！")

            # 特殊转化 廿
            inputs = inputs.replace("廿", "二十")

            # 检查输入数据是否有效
            sign, integer_data, decimal_data, is_all_num = self.__check_input_data_is_valid(inputs, mode)

            # smart 下的特殊情况
            if sign == 0:
                return integer_data
            else:
                if not is_all_num:
                    if decimal_data is None:
                        output = self.__integer_convert(integer_data)
                    else:
                        output = self.__integer_convert(integer_data) + self.__decimal_convert(decimal_data)
                        # fix 1 + 0.57 = 1.5699999999999998
                        output = round(output, len(decimal_data))
                else:
                    if decimal_data is None:
                        output = self.__direct_convert(integer_data)
                    else:
                        output = self.__direct_convert(integer_data) + self.__decimal_convert(decimal_data)
                        # fix 1 + 0.57 = 1.5699999999999998
                        output = round(output, len(decimal_data))
        else:
            raise ValueError("输入数据为空！")

        return sign * output

    def __get_pattern(self) -> dict:
        # 整数严格检查
        _0 = "[零]"
        _1_9 = f"[一二三四五六七八九]"
        _10_99 = f"{_1_9}?[十]{_1_9}?"
        _1_99 = f"({_10_99}|{_1_9})"
        _100_999 = f"({_1_9}[百]([零]{_1_9})?|{_1_9}[百]{_10_99})"
        _1_999 = f"({_100_999}|{_1_99})"
        _1000_9999 = f"({_1_9}[千]([零]{_1_99})?|{_1_9}[千]{_100_999})"
        _1_9999 = f"({_1000_9999}|{_1_999})"
        _10000_99999999 = f"({_1_9999}[万]([零]{_1_999})?|{_1_9999}[万]{_1000_9999})"
        _1_99999999 = f"({_10000_99999999}|{_1_9999})"
        _100000000_9999999999999999 = f"({_1_99999999}[亿]([零]{_1_99999999})?|{_1_99999999}[亿]{_10000_99999999})"
        _1_9999999999999999 = f"({_100000000_9999999999999999}|{_1_99999999})"
        str_int_pattern = f"^({_0}|{_1_9999999999999999})$"
        nor_int_pattern = f"^({_0}|{_1_9999999999999999})$"

        str_dec_pattern = "^[零一二三四五六七八九]{0,15}[一二三四五六七八九]$"
        nor_dec_pattern = "^[零一二三四五六七八九]{0,16}$"

        for str_num in self.strict_cn_number.keys():
            str_int_pattern = str_int_pattern.replace(str_num, self.strict_cn_number[str_num])
            str_dec_pattern = str_dec_pattern.replace(str_num, self.strict_cn_number[str_num])
        for nor_num in self.normal_cn_number.keys():
            nor_int_pattern = nor_int_pattern.replace(nor_num, self.normal_cn_number[nor_num])
            nor_dec_pattern = nor_dec_pattern.replace(nor_num, self.normal_cn_number[nor_num])

        pattern_dict = {
            "strict": {
                "int": str_int_pattern,
                "dec": str_dec_pattern
            },
            "normal": {
                "int": nor_int_pattern,
                "dec": nor_dec_pattern
            }
        }
        return pattern_dict

    def __copy_num(self, num):
        cn_num = ""
        for n in num:
            cn_num += self.conf["number_low_an2cn"][int(n)]
        return cn_num

    def __check_input_data_is_valid(self, check_data: str, mode: str) -> (int, str, str, bool):
        # 去除 元整、圆整
        check_data = check_data.replace("元整", "").replace("圆整", "")

        for data in check_data:
            if data not in self.check_key_dict[mode]:
                raise ValueError(f"当前为{mode}模式，输入的数据不在转化范围内：{data}！")

        # 确定正负号
        if check_data[0] == "负":
            check_data = check_data[1:]
            sign = -1
        else:
            sign = 1

        if "点" in check_data:
            split_data = check_data.split("点")
            if len(split_data) == 2:
                integer_data, decimal_data = split_data
                # 将 smart 模式中的阿拉伯数字转化成中文数字
                if mode == "smart":
                    integer_data = re.sub(r"\d+", lambda x: self.ac.an2cn(x.group()), integer_data)
                    decimal_data = re.sub(r"\d+", lambda x: self.__copy_num(x.group()), decimal_data)
                    mode = "normal"
            else:
                raise ValueError("数据中包含不止一个点！")
        else:
            integer_data = check_data
            decimal_data = None
            # 将 smart 模式中的阿拉伯数字转化成中文数字
            if mode == "smart":
                # 10.1万
                pattern1 = re.compile(fr"^-?\d+(\.\d+)?[{self.all_unit}]$")
                result1 = pattern1.search(integer_data)
                if result1:
                    if result1.group() == integer_data:
                        output = int(float(integer_data[:-1]) * self.conf["unit_cn2an"][integer_data[-1]])
                        return 0, output, None, None

                integer_data = re.sub(r"\d+", lambda x: self.ac.an2cn(x.group()), integer_data)
                mode = "normal"

        result_int = re.compile(self.pattern_dict[mode]["int"]).search(integer_data)

        if result_int:
            if result_int.group() == integer_data:
                if decimal_data is not None:
                    result_dec = re.compile(self.pattern_dict[mode]["dec"]).search(decimal_data)
                    if result_dec:
                        if result_dec.group() == decimal_data:
                            return sign, integer_data, decimal_data, False
                else:
                    return sign, integer_data, decimal_data, False
        else:
            if mode == "strict":
                raise ValueError(f"不符合格式的数据：{integer_data}")
            elif mode == "normal":
                # 纯数模式：一二三
                ptn_all_num = re.compile(f"^[{self.all_num}]+$")
                result_all_num = ptn_all_num.search(integer_data)
                if result_all_num:
                    if result_all_num.group() == integer_data:
                        if decimal_data is not None:
                            result_dec = re.compile(self.pattern_dict[mode]["dec"]).search(decimal_data)
                            if result_dec:
                                if result_dec.group() == decimal_data:
                                    return sign, integer_data, decimal_data, True
                        else:
                            return sign, integer_data, decimal_data, True

                # 口语模式：一万二，两千三，三百四
                ptn_speaking_mode = re.compile(f"^[{self.all_num}][{self.all_unit}][{self.all_num}]$")
                result_speaking_mode = ptn_speaking_mode.search(integer_data)
                if result_speaking_mode:
                    if result_speaking_mode.group() == integer_data:
                        _unit = self.conf["unit_low_an2cn"][self.conf["unit_cn2an"][integer_data[1]] // 10]
                        integer_data = integer_data + _unit
                        if decimal_data is not None:
                            result_dec = re.compile(self.pattern_dict[mode]["dec"]).search(decimal_data)
                            if result_dec:
                                if result_dec.group() == decimal_data:
                                    return sign, integer_data, decimal_data, False
                        else:
                            return sign, integer_data, decimal_data, False

        raise ValueError(f"不符合格式的数据：{check_data}")

    def __integer_convert(self, integer_data: str) -> int:
        # 核心
        output_integer = 0
        unit = 1
        ten_thousand_unit = 1
        for index, cn_num in enumerate(reversed(integer_data)):
            # 数值
            if cn_num in self.conf["number_cn2an"]:
                num = self.conf["number_cn2an"][cn_num]
                output_integer += num * unit
            # 单位
            elif cn_num in self.conf["unit_cn2an"]:
                unit = self.conf["unit_cn2an"][cn_num]
                # 判断出万、亿、万亿
                if unit % 10000 == 0:
                    # 万 亿
                    if unit > ten_thousand_unit:
                        ten_thousand_unit = unit
                    # 万亿
                    else:
                        ten_thousand_unit = unit * ten_thousand_unit
                        unit = ten_thousand_unit

                if unit < ten_thousand_unit:
                    unit = unit * ten_thousand_unit

                if index == len(integer_data) - 1:
                    output_integer += unit
            else:
                raise ValueError(f"{cn_num} 不在转化范围内")

        return int(output_integer)

    def __decimal_convert(self, decimal_data: str) -> float:
        len_decimal_data = len(decimal_data)

        if len_decimal_data > 16:
            print(f"注意：小数部分长度为 {len_decimal_data} ，将自动截取前 16 位有效精度！")
            decimal_data = decimal_data[:16]
            len_decimal_data = 16

        output_decimal = 0
        for index in range(len(decimal_data) - 1, -1, -1):
            unit_key = self.conf["number_cn2an"][decimal_data[index]]
            output_decimal += unit_key * 10 ** -(index + 1)

        # 处理精度溢出问题
        output_decimal = round(output_decimal, len_decimal_data)

        return output_decimal

    def __direct_convert(self, data: str) -> int:
        output_data = 0
        for index in range(len(data) - 1, -1, -1):
            unit_key = self.conf["number_cn2an"][data[index]]
            output_data += unit_key * 10 ** (len(data) - index - 1)

        return output_data


class An2Cn(object):
    def __init__(self) -> None:
        self.conf = get_default_conf()
        self.all_num = "0123456789"
        self.number_low = self.conf["number_low_an2cn"]
        self.number_up = self.conf["number_up_an2cn"]
        self.mode_list = ["low", "up", "rmb", "direct"]

    def an2cn(self, inputs: Union[str, int, float] = None, mode: str = "low") -> str:
        """
        阿拉伯数字转中文数字
        :param inputs: 阿拉伯数字
        :param mode: 小写数字，大写数字，人民币大写，直接转化
        :return: 中文数字
        """
        if inputs is not None and inputs != "":
            if mode not in self.mode_list:
                raise ValueError(f"mode 仅支持 {str(self.mode_list)} ！")

            # 将数字转化为字符串，这里会有Python会自动做转化
            # 1. -> 1.0 1.00 -> 1.0 -0 -> 0
            if not isinstance(inputs, str):
                inputs = self.__number_to_string(inputs)

            # 将全角数字和符号转化为半角
            inputs = self.__full_to_half(inputs)

            # 检查数据是否有效
            self.__check_inputs_is_valid(inputs)

            # 判断正负
            if inputs[0] == "-":
                sign = "负"
                inputs = inputs[1:]
            else:
                sign = ""

            if mode == "direct":
                output = self.__direct_convert(inputs)
            else:
                # 切割整数部分和小数部分
                split_result = inputs.split(".")
                len_split_result = len(split_result)
                if len_split_result == 1:
                    # 不包含小数的输入
                    integer_data = split_result[0]
                    if mode == "rmb":
                        output = self.__integer_convert(integer_data, "up") + "元整"
                    else:
                        output = self.__integer_convert(integer_data, mode)
                elif len_split_result == 2:
                    # 包含小数的输入
                    integer_data, decimal_data = split_result
                    if mode == "rmb":
                        int_data = self.__integer_convert(integer_data, "up")
                        dec_data = self.__decimal_convert(decimal_data, "up")
                        len_dec_data = len(dec_data)

                        if len_dec_data == 0:
                            output = int_data + "元整"
                        elif len_dec_data == 1:
                            raise ValueError(f"异常输出：{dec_data}")
                        elif len_dec_data == 2:
                            if dec_data[1] != "零":
                                if int_data == "零":
                                    output = dec_data[1] + "角"
                                else:
                                    output = int_data + "元" + dec_data[1] + "角"
                            else:
                                output = int_data + "元整"
                        else:
                            if dec_data[1] != "零":
                                if dec_data[2] != "零":
                                    if int_data == "零":
                                        output = dec_data[1] + "角" + dec_data[2] + "分"
                                    else:
                                        output = int_data + "元" + dec_data[1] + "角" + dec_data[2] + "分"
                                else:
                                    if int_data == "零":
                                        output = dec_data[1] + "角"
                                    else:
                                        output = int_data + "元" + dec_data[1] + "角"
                            else:
                                if dec_data[2] != "零":
                                    if int_data == "零":
                                        output = dec_data[2] + "分"
                                    else:
                                        output = int_data + "元" + "零" + dec_data[2] + "分"
                                else:
                                    output = int_data + "元整"
                    else:
                        output = self.__integer_convert(integer_data, mode) + self.__decimal_convert(decimal_data, mode)
                else:
                    raise ValueError(f"输入格式错误：{inputs}！")
        else:
            raise ValueError("输入数据为空！")

        return sign + output

    def __direct_convert(self, inputs: str) -> str:
        _output = ""
        for d in inputs:
            if d == ".":
                _output += "点"
            else:
                _output += self.number_low[int(d)]
        return _output

    @staticmethod
    def __number_to_string(number_data: Union[int, float]) -> str:
        # 小数处理：python 会自动把 0.00005 转化成 5e-05，因此 str(0.00005) != "0.00005"
        string_data = str(number_data)
        if "e" in string_data:
            string_data_list = string_data.split("e")
            string_key = string_data_list[0]
            string_value = string_data_list[1]
            if string_value[0] == "-":
                string_data = "0." + "0" * (int(string_value[1:]) - 1) + string_key
            else:
                string_data = string_key + "0" * int(string_value)
        return string_data

    @staticmethod
    def __full_to_half(inputs: str) -> str:
        # 全角转半角
        full_inputs = ""
        for uchar in inputs:
            inside_code = ord(uchar)
            # 全角空格直接转换
            if inside_code == 12288:
                full_inputs += chr(32)
            # 全角字符（除空格）根据关系转化
            elif 65281 <= inside_code <= 65374:
                full_inputs += chr(inside_code - 65248)
            else:
                full_inputs += uchar
        return full_inputs

    def __check_inputs_is_valid(self, check_data: str) -> None:
        # 检查输入数据是否在规定的字典中
        all_check_keys = self.all_num + ".-"
        for data in check_data:
            if data not in all_check_keys:
                raise ValueError(f"输入的数据不在转化范围内：{data}！")

    def __integer_convert(self, integer_data: str, mode: str) -> str:
        numeral_list = self.conf[f"number_{mode}_an2cn"]
        unit_list = self.conf[f"unit_{mode}_order_an2cn"]

        # 去除前面的 0，比如 007 => 7
        integer_data = str(int(integer_data))

        len_integer_data = len(integer_data)
        if len_integer_data > len(unit_list):
            raise ValueError(f"超出数据范围，最长支持 {len(unit_list)} 位")

        output_an = ""
        for i, d in enumerate(integer_data):
            if int(d):
                output_an += numeral_list[int(d)] + unit_list[len_integer_data - i - 1]
            else:
                if not (len_integer_data - i - 1) % 4:
                    output_an += numeral_list[int(d)] + unit_list[len_integer_data - i - 1]

                if i > 0 and not output_an[-1] == "零":
                    output_an += numeral_list[int(d)]

        output_an = output_an.replace("零零", "零").replace("零万", "万").replace("零亿", "亿").replace("亿万", "亿") \
            .strip("零")

        # 解决「一十几」问题
        if output_an[:2] in ["一十"]:
            output_an = output_an[1:]

        # 0 - 1 之间的小数
        if not output_an:
            output_an = "零"

        return output_an

    def __decimal_convert(self, decimal_data: str, o_mode: str) -> str:
        len_decimal_data = len(decimal_data)

        if len_decimal_data > 16:
            print(f"注意：小数部分长度为 {len_decimal_data} ，将自动截取前 16 位有效精度！")
            decimal_data = decimal_data[:16]

        if len_decimal_data:
            output_an = "点"
        else:
            output_an = ""
        numeral_list = self.conf[f"number_{o_mode}_an2cn"]

        for data in decimal_data:
            output_an += numeral_list[int(data)]
        return output_an


class Transform(object):
    def __init__(self) -> None:
        self.conf = get_default_conf()
        self.all_num = "零一二三四五六七八九"
        self.all_unit = "".join(list(self.conf["unit_cn2an"].keys()))
        self.cn2an = Cn2An().cn2an
        self.an2cn = An2Cn().an2cn
        self.cn_pattern = f"负?([{self.all_num}{self.all_unit}]+点)?[{self.all_num}{self.all_unit}]+"
        self.smart_cn_pattern = f"-?([0-9]+.)?[0-9]+[{self.all_unit}]+"

    def transform(self, inputs: str, method: str = "cn2an") -> str:
        inputs = inputs.replace("廿", "二十")
        if method == "cn2an":
            # date
            inputs = re.sub(
                fr"((({self.smart_cn_pattern})|({self.cn_pattern}))年)?([{self.all_num}十]+月)?([{self.all_num}十]+日)?",
                lambda x: self.__sub_util(x.group(), "cn2an", "date"), inputs)
            # fraction
            inputs = re.sub(fr"{self.cn_pattern}分之{self.cn_pattern}",
                            lambda x: self.__sub_util(x.group(), "cn2an", "fraction"), inputs)
            # percent
            inputs = re.sub(fr"百分之{self.cn_pattern}",
                            lambda x: self.__sub_util(x.group(), "cn2an", "percent"), inputs)
            # celsius
            inputs = re.sub(fr"{self.cn_pattern}摄氏度",
                            lambda x: self.__sub_util(x.group(), "cn2an", "celsius"), inputs)
            # number
            output = re.sub(self.cn_pattern,
                            lambda x: self.__sub_util(x.group(), "cn2an", "number"), inputs)
        elif method == "an2cn":
            # date
            inputs = re.sub(r"(\d{2,4}年)?(\d{1,2}月)?(\d{1,2}日)?",
                            lambda x: self.__sub_util(x.group(), "an2cn", "date"), inputs)
            # fraction
            inputs = re.sub(r"\d+/\d+",
                            lambda x: self.__sub_util(x.group(), "an2cn", "fraction"), inputs)
            # percent
            inputs = re.sub(r"-?(\d+\.)?\d+%",
                            lambda x: self.__sub_util(x.group(), "an2cn", "percent"), inputs)
            # celsius
            inputs = re.sub(r"\d+℃",
                            lambda x: self.__sub_util(x.group(), "an2cn", "celsius"), inputs)
            # number
            output = re.sub(r"-?(\d+\.)?\d+",
                            lambda x: self.__sub_util(x.group(), "an2cn", "number"), inputs)
        else:
            raise ValueError(f"error method: {method}, only support 'cn2an' and 'an2cn'!")

        return output

    def __sub_util(self, inputs, method: str = "cn2an", sub_mode: str = "number") -> str:
        try:
            if inputs:
                if method == "cn2an":
                    if sub_mode == "date":
                        return re.sub(fr"(({self.smart_cn_pattern})|({self.cn_pattern}))",
                                      lambda x: str(self.cn2an(x.group(), "smart")), inputs)
                    elif sub_mode == "fraction":
                        if inputs[0] != "百":
                            frac_result = re.sub(self.cn_pattern,
                                                 lambda x: str(self.cn2an(x.group(), "smart")), inputs)
                            numerator, denominator = frac_result.split("分之")
                            return f"{denominator}/{numerator}"
                        else:
                            return inputs
                    elif sub_mode == "percent":
                        return re.sub(f"(?<=百分之){self.cn_pattern}",
                                      lambda x: str(self.cn2an(x.group(), "smart")), inputs).replace("百分之", "") + "%"
                    elif sub_mode == "celsius":
                        return re.sub(f"{self.cn_pattern}(?=摄氏度)",
                                      lambda x: str(self.cn2an(x.group(), "smart")), inputs).replace("摄氏度", "℃")
                    elif sub_mode == "number":
                        return str(self.cn2an(inputs, "smart"))
                    else:
                        raise Exception(f"error sub_mode: {sub_mode} !")
                else:
                    if sub_mode == "date":
                        inputs = re.sub(r"\d+(?=年)",
                                        lambda x: self.an2cn(x.group(), "direct"), inputs)
                        return re.sub(r"\d+",
                                      lambda x: self.an2cn(x.group(), "low"), inputs)
                    elif sub_mode == "fraction":
                        frac_result = re.sub(r"\d+", lambda x: self.an2cn(x.group(), "low"), inputs)
                        numerator, denominator = frac_result.split("/")
                        return f"{denominator}分之{numerator}"
                    elif sub_mode == "celsius":
                        return self.an2cn(inputs[:-1], "low") + "摄氏度"
                    elif sub_mode == "percent":
                        return "百分之" + self.an2cn(inputs[:-1], "low")
                    elif sub_mode == "number":
                        return self.an2cn(inputs, "low")
                    else:
                        raise Exception(f"error sub_mode: {sub_mode} !")
        except Exception as e:
            print(f"WARN: {e}")
            return inputs


an2Cn = Transform().transform

if __name__ == '__main__':
    pass
    testStr = ['罚款2000元', '罚款两千元', '罚款俩千元整', '罚款人民币两千元',
               '《中华人民共和国水污染防治法》第八十三条第(二)项，并处十万元以上一百万元以下的罚款',
               '十一', '一百二十三', '两百三十二', '一千二百零三', '一万一千一百零一', u'十万零三千六百零九', u'一百二十三万四千五百六十七',
               u'一千一百二十三万四千五百六十七', u'一亿一千一百二十三万四千五百六十七', u'一百零二亿五千零一万零一千零三十八',
               u'一千一百一十一亿一千一百二十三万四千五百六十七', u'一兆一千一百一十一亿一千一百二十三万四千五百六十七',
               '我有百三十二块钱', '十二个套餐', '一亿零八万零三百二十三', '今天天气真不错',
               '百分之八十 discount rate很高了', '千万不要', '我们俩个人', '这个invoice value值一百万',
               '我的一百件商品have quality', '找一找我的收藏夹里，有没有一个眼镜']
    for tstr in testStr:
        print(tstr + ' = ' + ChineseNumToArab(tstr))

    output = an2Cn("消防法第2条", "an2cn")
    print(output)

    output = an2Cn("消防法第十五条", "cn2an")
    print(output)
